// gsms.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#include "gsms.h"
#include <qdom.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qsettings.h>
#include <qdir.h>
#include <qtimer.h>

const QString SETTINGS_DIR = "/.aabrowse/";
const QString APP_KEY = "/aabrowse/";

Gsms::Gsms( int serverTimeout )
   : QSocket( )
{
   bytes = 0;
   connect ( this, SIGNAL ( connected() ),
      this, SLOT ( sendPacket()) );
   connect ( this, SIGNAL ( readyRead() ),
      this, SLOT ( readXML()) );
   connect ( this, SIGNAL ( connectionClosed()),
      this, SLOT ( finished()));
   timer = new QTimer ( this );
   connect ( timer, SIGNAL ( timeout()), this, SLOT ( timeout()));
   timer->start ( serverTimeout * 1000, TRUE );
}

//-------------------------------------------------------------------
//  Set the packet type when query is started
//-------------------------------------------------------------------
void Gsms::setPacketType ( QString type )
{
   packetTypeText = type;
}

//-------------------------------------------------------------------
//  Set GUID of a specific server when doing an INFO query
//-------------------------------------------------------------------
void Gsms::setGuid ( QString guid )
{
   guidText = guid;
}

//-------------------------------------------------------------------
//  Send initial binary data then send XML data
//-------------------------------------------------------------------
void Gsms::sendPacket()
{
   //  Master server expects this data first. 2 bytes used to specify size
   //  of XML packet.
   char masterData[20] = { 0x47,0x53,0x4d,0x53,0x30,0x31,0x30,0x30,0x00,0x00,
      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 };
   QString packet;
   if ( packetTypeText == "COUNT" )
      packet = createPacket ( COUNT );
   if ( packetTypeText == "LIST" )
      packet = createPacket ( LIST );
   if ( packetTypeText == "INFO" )
      packet = createPacket ( INFO );
   //  Convert int size into binary
   decToBin ( masterData, packet.length());
   writeBlock ( masterData, 20 );
   flush();
   writeBlock ( packet, packet.length() );
}

//-------------------------------------------------------------------
//  Creates an XML doc for each type of packet then returns it as a QString
//-------------------------------------------------------------------
QString Gsms::createPacket( packetType pt )
{
   QStringList fieldList;
   fieldList.append ( "OFFICIAL" );
   fieldList.append ( "LEASED" );
   fieldList.append ( "TOUR" );
   fieldList.append ( "MAXHONOR" );
   fieldList.append ( "MINHONOR" );
   fieldList.append ( "MILES" );
   fieldList.append ( "SV_PUNKBUSTER" );
   fieldList.append ( "CHEATS" );
   fieldList.append ( "TOURNAMENT" );
   fieldList.append ( "ULTIMATEARENA" );
   fieldList.append ( "CUSTOMFLAGS" );

   QStringList filterList = buildFilterList ();

   QDomDocument doc ( "packet" );
   // Header
   QString docText ( "<?xml version=\"1.0\"?>" );
   doc.setContent ( docText );
   // Doc root
   QDomElement root = doc.createElement ( "GSMS" );
   doc.appendChild ( root );
   QDomElement messages = doc.createElement ( "MESSAGES" );
   root.appendChild ( messages );
   QDomElement message = doc.createElement ( "MESSAGE" );
   messages.appendChild ( message );

   // Action - specifies type of packet
   QDomElement action = doc.createElement ( "ACTION" );
   message.appendChild ( action );

   QDomText actionText;
   if ( pt == COUNT )
      actionText = doc.createTextNode( "GETSERVERCOUNT" );

   if ( pt == LIST )
      actionText = doc.createTextNode( "GETSERVERLIST" );

   if ( pt == INFO )
      actionText = doc.createTextNode( "GETSERVERINFO" );

   action.appendChild ( actionText );

   // Server info packet. Send GUID of server we need info for
   if ( pt == INFO )
   {
      QDomElement guid = doc.createElement ( "GUID" );
      message.appendChild ( guid );

      QDomText guidTextNode = doc.createTextNode ( guidText );
      guid.appendChild ( guidTextNode );
   }

   // List and Count packets
   else
   {
      QDomElement servertype = doc.createElement ( "SERVERTYPE" );
      message.appendChild ( servertype );

      QDomText servertypeText = doc.createTextNode( "AAO" );
      servertype.appendChild ( servertypeText );
   }
   // List packet - append list of fields for each type of filter
   if ( pt == LIST )
   {
      QDomElement page = doc.createElement ( "PAGE" );
      message.appendChild ( page );
      QDomElement pagesize = doc.createElement ( "PAGESIZE" );
      message.appendChild ( pagesize );
      QDomElement fields = doc.createElement ( "FIELDS" );
      message.appendChild ( fields );
      QDomText pageText = doc.createTextNode ( "0" );
      page.appendChild ( pageText );
      QDomText pageSizeText = doc.createTextNode ( "999999" );
      pagesize.appendChild ( pageSizeText );

      for ( QStringList::Iterator it = fieldList.begin(); it != fieldList.end();
         ++it )
      {
         QDomElement field = doc.createElement ( "FIELD" );
	 fields.appendChild ( field );
	 QDomElement name = doc.createElement ( "NAME" );
	 field.appendChild ( name );
         QDomText fieldText = doc.createTextNode ( *it );
	 name.appendChild ( fieldText );
      }
   }
   // Create server filters. Only minhonor, maxhonr, map and version work
   if ( pt == LIST || pt == COUNT && !filterList.isEmpty() )
   {
      QDomElement filters = doc.createElement ( "FILTERS" );
      message.appendChild ( filters );

      // Load server filter settings
      QSettings settings;
      settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() + 
         SETTINGS_DIR );
      int minHonor = settings.readNumEntry ( APP_KEY + "Filters/" + 
         "minhonor", 1 );
      int maxHonor = settings.readNumEntry ( APP_KEY + "Filters/" + 
         "maxhonor", 100 );
      int minPlayers = settings.readNumEntry ( APP_KEY + "Filters/" +
         "MBS_PLAYERS", 0 );
      int maxPlayers = settings.readNumEntry ( APP_KEY + "Filters/" +
         "MBS_MAXPLAYERS", 26 );
      QString map = settings.readEntry ( APP_KEY + "Filters/" + "map" );
      QString type = settings.readEntry ( APP_KEY + "Filters/" + "type" );

      for ( QStringList::Iterator it = filterList.begin(); 
         it != filterList.end(); ++it )
      {
         QDomElement field = doc.createElement ( "FIELD" );
         filters.appendChild ( field );

         QDomElement numeric = doc.createElement ( "NUMERIC" );
         field.appendChild ( numeric );
	 QDomText numericText;
	 if ( *it == "maxhonor" || *it == "minhonor" || *it == "MBS_PLAYERS"
	    || *it == "MBS_MAXPLAYERS" )
            numericText = doc.createTextNode ( "1" );
	 else
	    numericText = doc.createTextNode ( "0" );
         numeric.appendChild ( numericText );
	 
         QDomElement name = doc.createElement ( "NAME" );
         field.appendChild ( name );
         QDomText nameText = doc.createTextNode ( *it );
         name.appendChild ( nameText );

         QDomElement filterType  = doc.createElement ( "FILTERTYPE" );
         field.appendChild ( filterType );
	 QDomText filterTypeText;
	 if ( *it == "minhonor" || *it == "MBS_PLAYERS" )
	    filterTypeText = doc.createTextNode 
	       ( "GREATERTHANOREQUAL" );
	 else
	 {
	    if ( *it == "maxhonor" || *it == "MBS_MAXPLAYERS" )
	       filterTypeText = doc.createTextNode ( "LESSTHANOREQUAL" );
	    else
               filterTypeText = doc.createTextNode ( "EQUAL" );
	 }
         filterType.appendChild ( filterTypeText );

         QDomElement value = doc.createElement ( "VALUE" );
         field.appendChild ( value );
	 QDomText valueText;

	 if ( *it == "MBS_MAP" )
	 {
	    if ( !map.isEmpty()  )
               valueText = doc.createTextNode ( map );
	 }
	 if ( *it == "minhonor" )
	    valueText = doc.createTextNode ( QString::number (minHonor) );
	 if ( *it == "maxhonor" )
	    valueText = doc.createTextNode ( QString::number (maxHonor) );
	 if ( *it == "MBS_PLAYERS" )
	    valueText = doc.createTextNode ( QString::number (minPlayers) );
	 if ( *it == "MBS_MAXPLAYERS" )
	    valueText = doc.createTextNode ( QString::number (maxPlayers) );
	 if ( *it == "official" )
	 {
	    if ( type == "Leased" || type == "Authorized" )
	       valueText = doc.createTextNode ( "0" );
	    else
	       valueText = doc.createTextNode ( "1" );
	 }
	 if ( *it == "leased" )
	 {
	    if ( type == "Authorized" || type == "Official" )
	       valueText = doc.createTextNode ( "0" );
	    else
	       valueText = doc.createTextNode ( "1" );
	 }
	 if ( *it == "miles" )
	    valueText = doc.createTextNode ( "0" );
	 if ( *it == "MBS_PRIVATE" )
	    valueText = doc.createTextNode ( "0" );
	 if ( *it == "cheats" )
	    valueText = doc.createTextNode ( "0" );
         value.appendChild ( valueText );
      }
   }
   QString xml = doc.toString(0);
   // Remove all spaces and new lines from string
   xml = xml.simplifyWhiteSpace();
   return xml;
}

QStringList Gsms::buildFilterList()
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() +
      SETTINGS_DIR );
   int minHonor = settings.readNumEntry ( APP_KEY + "Filters/" +
      "minhonor", 1 );
   int maxHonor = settings.readNumEntry ( APP_KEY + "Filters/" +
      "maxhonor", 100 );
   int minPlayers = settings.readNumEntry ( APP_KEY + "Filters/" +
      "MBS_PLAYERS", 0 );
   int maxPlayers = settings.readNumEntry ( APP_KEY + "Filters/" +
      "MBS_MAXPLAYERS", 26 );
   QString map = settings.readEntry ( APP_KEY + "Filters/" + "map" );
   QString type = settings.readEntry ( APP_KEY + "Filters/" + "type" );
   bool miles = settings.readBoolEntry ( APP_KEY + "Filters/" + "miles", 1 );
   bool cheat = settings.readBoolEntry ( APP_KEY + "Filters/" + "cheats", 0 );
   bool locked = settings.readBoolEntry ( APP_KEY + "Filters/" + 
      "MBS_PRIVATE", 0 );

   QStringList filterList;
   if ( minHonor != 1 )
      filterList.append ( "minhonor" );
   if ( maxHonor != 100 )
      filterList.append ( "maxhonor" );
   if ( minPlayers != 0 )
      filterList.append ( "MBS_PLAYERS" );
   if ( maxPlayers != 26 )
      filterList.append ( "MBS_MAXPLAYERS" );
   if ( !map.isEmpty() )
      filterList.append ( "MBS_MAP" );
   if ( !type.isEmpty())
   {
      if ( type != "All Honor" )
         filterList.append  ( "leased" );
      filterList.append  ( "official" );
   }
   if ( !miles )
      filterList.append ( "miles" );
   if ( !cheat )
      filterList.append ( "cheats" );
   if ( !locked )
      filterList.append ( "MBS_PRIVATE" );
   return filterList;
}

//-------------------------------------------------------------------
//  Reads the XML data from the socket.
//-------------------------------------------------------------------
void Gsms::readXML()
{
   timer->stop();
   int length = 0;
   int numServers;
   QDomDocument doc ( "receivedpacket" );

   length = bytesAvailable();

   QCString data;
   data.resize ( length + 1 );
   readBlock ( data.data(), length );
   data[length] = '\0';
   if ( data[length-1] == 10 && data[length-2] == 13 )
      data[length-2] = '\0';

   if ( data.left(13) == "<?xml version" )
   {

      doc.setContent ( data );
      QDomNodeList list = doc.elementsByTagName ( "ACTION" );
      QDomNode node = list.item ( 0 );
      QDomElement actionElement = node.toElement();
      QString action = actionElement.text();

      if ( action == "GETSERVERCOUNT" )
      {
	 if ( packetTypeText == "COUNT" )
	 {
            QDomNodeList countList = doc.elementsByTagName ( "COUNT" );
            QDomNode countNode = countList.item ( 0 );
            QDomElement countElement = countNode.toElement();
            QString numString = countElement.text();
            numServers = numString.toInt();
            emit numberOfServers ( numServers );
            if ( numServers > 0 )
            {
               packetTypeText = "LIST";
               sendPacket();
	       return;
            }
	 }
	 else if ( packetTypeText == "LIST" )
	 {
            bytes = bytes + length;
	    emit bytesRead ( bytes );
	    finalString = data.data();
	 }
      }
   }
   else if ( data.left(8) != "GSMS0100" )
   {
      bytes = bytes + length;
      emit bytesRead ( bytes );
      finalString.append ( data.data());
   }
}

//-------------------------------------------------------------------
//  Returns the completed serverList as a QString
//-------------------------------------------------------------------
void Gsms::finished()
{
   emit serverList ( finalString );
}

void Gsms::timeout()
{
   close();
   emit serverTimeout();
}

//-------------------------------------------------------------------
// Convert decimal packet length into 2 binary chars and update the packet data
//-------------------------------------------------------------------
void Gsms::decToBin ( char *data, int dec )
{
   unsigned char firstBytes = 0, lastBytes = 0;
   for ( int i = 15; i >= 0; i-- )
   {
      unsigned long temp_value;
      temp_value = dec / (1 << i); // Devide by 2^i
      if ( temp_value > 0 )
         dec = dec % (1 << i);
      if ( i > 7 )
         firstBytes = firstBytes + (temp_value << (i - 8));
      else 
         lastBytes = lastBytes + (temp_value << i);
   }
   // Update master data with size of the next packet
   data[12] = lastBytes;
   data[16] = lastBytes;
   data[13] = firstBytes;
   data[17] = firstBytes;
}
