// gspyquery.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#include "gspyquery.h"
#include "serverlistview.h"

#include <qsocketdevice.h>
#include <qsocketnotifier.h>
#include <qregexp.h>
#include <qtimer.h>
#include <qdatetime.h>

GSpyQuery::GSpyQuery()
{
   pong = FALSE;
   totalPackets = 0, receivedPackets = 0;
   // Allocate space for 4 packets
   packets.reserve ( 4 );
}

GSpyQuery::~GSpyQuery()
{
   delete pingTimer;
}

//-------------------------------------------------------------------
//  Send gamespy UDP status packet to server
//-------------------------------------------------------------------
void GSpyQuery::sendQuery ( QString address )
{
   QHostAddress addr;
   QString port;
   
   addr.setAddress ( address.section ( ':', 0,0 ));
   port = address.section ( ':', 1,1 );

   pingTimer = new QTime ();
   pingTimer->start();
   
   qsd->writeBlock ( "\\status\\", 8, addr, port.toInt()+1);
}

//-------------------------------------------------------------------
//  Emits the final data string and emits the ping time when the first
//  packet arrives
//-------------------------------------------------------------------
void GSpyQuery::recvQuery (int)
{
   int ms = pingTimer->restart();
   timer->stop();
   QCString recvData;
   int bytes = qsd->bytesAvailable();
   if ( bytes >= 0 )
   {
      // Need extra space for NULL terminator
      recvData.resize ( bytes + 1 );
      qsd->readBlock ( recvData.data(), bytes );
   
      QString address = qsd->peerAddress().toString() + ':' + 
         QString::number ( qsd->peerPort() -1);

      QString data = recvData.data();
      int index;
      // Some servers have the packet index at the end, other don't
      if ( data.endsWith ( "final\\" ))
         index = data.mid ( data.length() - 8, 1 ).toInt();
      else
         index = data.right ( 1 ).toInt();
      // Assume only 1 packet if index can't be read
      if ( index <= 0 || index > 4 )
         index = 1;

      if ( !pong )
      {
         emit time ( ms, address );
         // Only update ping once incase the data is spread over 2 packets
         pong = TRUE;
      }
      //  Packet containing final holds the total number of packets
      if ( data.contains ( "\\final\\" ))
	 totalPackets = index;

      packets[index - 1] = data;
      receivedPackets++;
      // All packets received, emit the data
      if ( receivedPackets == totalPackets )
      {
         QCString finalString;
	 for ( int i = 0; i < totalPackets; i++ )
            finalString.append ( packets[i] );
	 parseQuery ( finalString );
      }
   }
}

//-------------------------------------------------------------------
//  Parse the gamespy data and update server and player values
//-------------------------------------------------------------------
void GSpyQuery::parseQuery ( QCString queryc )
{
   QString query = queryc;
   currList->setCurrentServer ( serverAddress );
   if ( !currList->isCurrentServerSet())
      return;
   currList->setServerBuddyIcon ( FALSE );
   while ( !query.isNull())
   {
      int indexStart = query.find ( '\\', 0, FALSE );
      int indexMiddle = query.find ( '\\', indexStart + 1, FALSE );
      int indexEnd = query.find ( '\\', indexMiddle + 1, FALSE );
      if ( indexEnd >= 0 && indexMiddle >= 0 )
      {
         QString pairString = query.mid ( indexStart +1,
	    indexEnd - indexStart -1 );
         QString value = pairString.section ( '\\', 1, 1 );


         if ( pairString.startsWith ( "hostname\\" ))
            currList->setServerName ( value );

         // Some servers return a mapname string, others maptitle
         if ( pairString.startsWith ( "maptitle\\"  ) ||
               pairString.startsWith ( "mapname\\" ))
            currList->setServerMap ( value );

         if ( pairString.startsWith ( "tour\\" ))
            currList->setServerTour ( value );

         if ( pairString.startsWith ( "official\\" ))
         {
            if ( value == "1" )
            {
               official = TRUE;
               // if leased it is not official but an ordinary honor server
               if ( leased == TRUE )
                  currList->setServerHonor ( TRUE );
               else
                  currList->setServerOfficial ( TRUE );
            }
            else if ( leased == FALSE )
               currList->setServerOfficial ( FALSE );
         }

         if ( pairString.startsWith ( "leased\\" ))
         {
            if ( value == "1" )
            {
               leased = TRUE;
               if ( official == TRUE )
                  currList->setServerHonor ( TRUE );
               else
                  currList->setServerLeased ( TRUE );
            }
            else if ( official == FALSE )
               currList->setServerLeased ( FALSE );
         }

         if ( pairString.startsWith ( "miles\\" ))
         {
            if ( value == "1" )
               currList->setServerMiles ( TRUE );
            else
               currList->setServerMiles ( FALSE );
         }

         if ( pairString.startsWith ( "password\\" ))
         {
            if ( value == "1")
               currList->setServerPrivate ( TRUE );
            else
               currList->setServerPrivate ( FALSE );
         }

         if ( pairString.startsWith ( "sv_punkbuster\\" ))
         {
            if ( value == "1" )
               currList->setServerPB ( TRUE );
            else
               currList->setServerPB ( FALSE );
         }

         if ( pairString.startsWith ( "maxplayers\\" ))
            currList->setServerMaxPlayers ( value );

         if ( pairString.startsWith ( "numplayers\\" ))
         {
            currList->setServerNumPlayers ( value );
            currList->setActualPlayerNumber ();
         }

         // workaround for bug that occurs when player just joins server
         if ( value.startsWith ( "player_" ) || value.startsWith ( "kia_" )
            || value.startsWith ( "score_" ) || value.startsWith ( "ping_" )
            || value.startsWith ( "roe_" ) || value.startsWith ( "leader_" )
            || value.startsWith ( "enemy_" ) || value.startsWith ( "goal_" ))
         {
            indexEnd = indexMiddle;
            pairString = query.mid ( indexStart +1,
	       indexMiddle - indexStart -1 );
            value = "";
         }

         if ( pairString.section ( '\\',0,0 ).contains (
            QRegExp ( "\\w_\\d\\d?$" )))
         {
            QString type = pairString.section ( '_', 0, 0 );
            QRegExp rx ( "(\\d\\d?)" );
            rx.search ( pairString );
            int playerNum = rx.cap ( 1 ).toInt();
            currList->setPlayerValue ( playerNum, type, value );
         }
         else if ( pairString.startsWith ( "gamever\\" ))
            currList->setSettingValue ( "Version", value );
         else if ( pairString.startsWith ( "cheats\\" ))
            currList->setSettingValue ( "Cheats", value );
         else if ( pairString.startsWith ( "minhonor\\" ))
            currList->setSettingValue ( "Minimum Honor", value );
         else if ( pairString.startsWith ( "maxhonor\\" ))
            currList->setSettingValue ( "Maximum Honor", value );
         else if ( pairString.startsWith ( "current_round\\" ))
            currList->setSettingValue ( "Current Round", value );
         else if ( pairString.startsWith ( "mission_time\\" ))
            currList->setSettingValue ( "Mission Time", value );
         else if ( pairString.startsWith ( "AdminEMail\\" ))
            currList->setSettingValue ( "Admin Email", value );
         else if ( pairString.startsWith ( "AdminName\\" ))
            currList->setSettingValue ( "Admin Name", value );
         else if ( pairString.startsWith ( "score_t0\\" ))
            currList->setSettingValue ( "Team 1 Score", value );
         else if ( pairString.startsWith ( "score_t1\\" ))
            currList->setSettingValue ( "Team 2 Score", value );
         query.remove ( indexStart, indexEnd - indexStart );
      }
      else
         break;
   }
   emit finished ( serverAddress );
}
