// settingsdialog.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#include "settingsdialog.h"

#include <qlineedit.h>
#include <qlayout.h>
#include <qhgroupbox.h>
#include <qvgroupbox.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qsettings.h>
#include <qlistview.h>
#include <qwidgetstack.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qcheckbox.h>
#include <qgrid.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qheader.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>

const QString SETTINGS_DIR = "/.aabrowse/";
const QString APP_KEY = "/aabrowse/";

SettingsDialog::SettingsDialog ( QWidget *parent=0, const char *name=0,
   bool modal=FALSE, WFlags f=0 )
   : QDialog ( parent, name, modal, f )
{
   mainBox = new QVBoxLayout ( this );
   mainBox->setMargin ( 5 );
   mainBox->setSpacing ( 5 );
   optionsBox = new QHBox ( this );
   optionsBox->setSpacing ( 5 );
   mainBox->addWidget ( optionsBox );
   
   selectList = new QListView ( optionsBox );
   selectList->addColumn ( "Options" );
   selectList->setSorting ( -1 );
   selectList->header()->hide();

   general = new QListViewItem ( selectList, "General" );
   andromeda = new QListViewItem ( selectList, general, "Master Server" );

   connect ( selectList, SIGNAL ( clicked ( QListViewItem * )), this,
      SLOT ( changeOptions ( QListViewItem *)));

   options = new QWidgetStack ( optionsBox );
   options->setMinimumSize ( QSize ( 400, 420 ));
   optionsGeneral = new QVBox ( options );
   optionsGeneral->setSpacing ( 10 );
   optionsAndro = new QVBox ( options );
   optionsAndro->setSpacing ( 10 );
   options->addWidget ( optionsGeneral, 0 );
   options->addWidget ( optionsAndro, 1 );
   options->raiseWidget ( 0 );

   // General options
   gameExecBox = new QHGroupBox ( "Location of AA executable",
      optionsGeneral );
   gameExecEdit = new QLineEdit ( gameExecBox );
   gameExecEdit->setMinimumWidth ( 200 );
   gameExecButton = new QPushButton ( "Browse", gameExecBox );

   startupBox = new QVGroupBox ( "Startup", optionsGeneral );
   refreshFavs = new QCheckBox ( startupBox );
   refreshFavs->setText ( "Refresh servers in favourites" );

   exitBox = new QVGroupBox ( "Exit", optionsGeneral );
   saveLists = new QCheckBox ( exitBox );
   saveLists->setText ( "Save server lists" );

   QHGroupBox *toolbar = new QHGroupBox ( "Toolbar", optionsGeneral );
   toolGroup = new QButtonGroup ();
   QRadioButton *bothButton = new QRadioButton ( "Icons and text", toolbar );
   QRadioButton *iconButton = new QRadioButton ( "Icons", toolbar );
   toolGroup->insert ( bothButton, 1 );
   toolGroup->insert ( iconButton, 2 );
   connectionBox = new QGrid ( 2, optionsGeneral );
   connectionBox->setSpacing ( 5 );
   
   openSocketsLabel = new QLabel ( "Maximum Sockets:", connectionBox );
   openSockets = new QSpinBox ( 5, 100, 5, connectionBox );
   timeoutLabel = new QLabel ( "Server Timeout:", connectionBox );
   timeout = new QSpinBox ( 1, 60, 1, connectionBox );
   retriesLabel = new QLabel ( "Number of Retries:", connectionBox );
   retries = new QSpinBox ( 0, 5, 1, connectionBox );

   gamespyCheckBox = new QCheckBox ( "Use new gamespy protocol", 
      optionsGeneral );
   colourNamesCheckBox = new QCheckBox ( "Display colored server names",
      optionsGeneral );
   
   // Andromeda options
   androBox = new QGrid ( 2, optionsAndro );
   masterGroup = new QButtonGroup ();

   QRadioButton *gspyButton = new QRadioButton ( "GameSpy", androBox );
   gspyEdit = new QLineEdit ( androBox );

   QRadioButton *androButton = new QRadioButton ( "Andromeda", androBox );
   androEdit = new QLineEdit ( androBox );

   QRadioButton *aamslButton = new QRadioButton ( "AAMSL", androBox );
   aamslEdit = new QLineEdit ( androBox );
   
   masterGroup->insert ( gspyButton, 1 );
   masterGroup->insert ( androButton, 2 );
   masterGroup->insert ( aamslButton, 3 );

   masterTimeoutLabel = new QLabel ( "Timeout:", androBox );
   masterTimeout = new QSpinBox ( 1, 60, 1, androBox );

   masterRefresh = new QCheckBox ( "Refresh servers after update",
      optionsAndro );
   
   controlBox = new QHBoxLayout ( mainBox );
   controlBox->addSpacing ( 200 );
   okButton = new QPushButton ( "OK", this );
   cancelButton = new QPushButton ( "Cancel", this );
   okButton->setDefault ( TRUE );

   controlBox->addWidget ( okButton );
   controlBox->addWidget ( cancelButton );

   connect ( gameExecButton, SIGNAL ( clicked ()), this, SLOT ( chooseExec ()));
   connect ( okButton, SIGNAL ( clicked ()), this, SLOT ( saveSettings()));
   connect ( cancelButton, SIGNAL ( clicked ()), this, SLOT ( reject()));
   loadSettings();
}

SettingsDialog::~SettingsDialog()
{
   delete masterGroup;
   delete toolGroup;
}

void SettingsDialog::changeOptions ( QListViewItem *newItem )
{
   if ( !newItem )
      return;
   if ( newItem->text (0) == "General" )
      options->raiseWidget (0);
   if ( newItem->text (0) == "Master Server" )
      options->raiseWidget (1);
}

void SettingsDialog::chooseExec ()
{
   QString exec;

   exec = QFileDialog::getOpenFileName ( QString::null, QString::null, this,
      "army exec dialog", "Select Americas Army Executable" );
   if ( exec )
      gameExecEdit->setText ( exec );
}

//-------------------------------------------------------------------
//  Load settings from file in home dir when dialog opened
//-------------------------------------------------------------------
void SettingsDialog::loadSettings()
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() + 
      SETTINGS_DIR );

   // General options
   gameExecEdit->setText ( settings.readEntry( APP_KEY + "Settings/" + 
      "army executable" ));
   refreshFavs->setChecked ( settings.readBoolEntry ( APP_KEY + "Settings/" +
      "refresh on startup", FALSE ));
   saveLists->setChecked ( settings.readBoolEntry ( APP_KEY + "Settings/" +
      "save lists on exit", TRUE ));
   openSockets->setValue ( settings.readNumEntry ( APP_KEY + "Settings/" +
      "sockets", 20 ));
   timeout->setValue ( settings.readNumEntry ( APP_KEY + "Settings/" +
      "refresh timeout", 3 ));
   retries->setValue ( settings.readNumEntry ( APP_KEY + "Settings/" +
      "retries", 1 ));
   gamespyCheckBox->setChecked ( settings.readBoolEntry ( APP_KEY +
      "Settings/" + "new gamespy protocol", TRUE ));
   colourNamesCheckBox->setChecked ( settings.readBoolEntry ( APP_KEY + 
      "Settings/" + "coloured names", TRUE ));
   toolGroup->setButton ( settings.readNumEntry ( APP_KEY + "Settings/" +
      "toolbutton type", 1 ));

   // Andromeda options
   gspyEdit->setText ( settings.readEntry ( APP_KEY + "Settings/" + 
      "gamespy master", "master.gamespy.com" ));
   androEdit->setText ( settings.readEntry ( APP_KEY + "Settings/" +
      "andro master", "master0.supercomputerinc.com" ));
   aamslEdit->setText ( settings.readEntry ( APP_KEY + "Settings/" +
      "aamsl master", "http://www.aamsl.net/index.php?" ));
   masterTimeout->setValue ( settings.readNumEntry ( APP_KEY + "Settings/" +
      "master timeout", 10 ));
   masterGroup->setButton ( settings.readNumEntry ( APP_KEY + "Settings/" +
      "master type", 1 ));
   masterRefresh->setChecked ( settings.readBoolEntry ( APP_KEY + "Settings/" +
      "master refresh", FALSE ));
}

//-------------------------------------------------------------------
//  Save settings when OK pressed
//-------------------------------------------------------------------
void SettingsDialog::saveSettings()
{
   QSettings *settings = new QSettings();
   settings->insertSearchPath ( QSettings::Unix, QDir::homeDirPath() +
      SETTINGS_DIR );

   // General options
   settings->writeEntry ( APP_KEY + "Settings/" + "army executable", 
      gameExecEdit->text());
   settings->writeEntry ( APP_KEY + "Settings/" + "refresh on startup",
      refreshFavs->isChecked());
   settings->writeEntry ( APP_KEY + "Settings/" + "save lists on exit",
      saveLists->isChecked());
   settings->writeEntry ( APP_KEY + "Settings/" + "sockets",
      openSockets->value());
   settings->writeEntry ( APP_KEY + "Settings/" + "refresh timeout",
      timeout->value());
   settings->writeEntry ( APP_KEY + "Settings/" + "retries",
      retries->value());
   settings->writeEntry ( APP_KEY + "Settings/" + "new gamespy protocol",
      gamespyCheckBox->isChecked());
   settings->writeEntry ( APP_KEY + "Settings/" + "coloured names",
      colourNamesCheckBox->isChecked());
   settings->writeEntry ( APP_KEY + "Settings/" + "toolbutton type",
      toolGroup->selectedId());

   // Andromeda options
   settings->writeEntry ( APP_KEY + "Settings/" + "gamespy master",
      gspyEdit->text());
   settings->writeEntry ( APP_KEY + "Settings/" + "andro master",
      androEdit->text());
   settings->writeEntry ( APP_KEY + "Settings/" + "aamsl master",
      aamslEdit->text());
   settings->writeEntry ( APP_KEY + "Settings/" + "master timeout",
      masterTimeout->value());
   settings->writeEntry ( APP_KEY + "Settings/" + "master type",
      masterGroup->selectedId());
   settings->writeEntry ( APP_KEY + "Settings/" + "master refresh",
      masterRefresh->isChecked());

   // Deleting writes the settings to disk
   delete settings;
   emit settingsChanged();
   accept();
}
